/*
   Test program of the generic tbrowse print facility.

   Compile:  Clipper Test /n /w /a

   Link:  RTLink fi Test, TBPrint

   Author:  Ed Stoll
   Copyright:  C.W. Kreimer & Associates, Inc.     1992

                                                               */

#include "SetCurs.ch"
#include "inkey.ch"
#include "Box.ch"

Function Main
	Local nOptions, cOldColor := setcolor()

	SetCursor( SC_NONE )

	Setcolor( 'W+/B, N/BG, N' )
	While .t.

		nOptions = Alert( ' ; Select TBrowse data type ; ', ;
                   {'Array', 'Database', 'Scoped Database' } )

		If nOptions == 1
			TBArray()
		Elseif nOptions == 2
			TBDbf()
      Elseif nOptions == 3
         TBScopeDbf()
		Elseif nOptions == 0
			Exit
		Endif

	End

	SetColor( cOldColor )
	SetCursor( SC_NORMAL )
	cls

Return Nil

///////////////////////////////////////////////////////////////////////////
Function TBDbf
   Local o, nKey
   Field ProjId, EmplId

   Use TimeData New
   Index on ProjId + EmplId to TimeData
   Go Top

	o := TBrowseDB( 6, 10, 19, 69 )

	o:HeadSep := ""
	o:ColSep  := " "

   o:AddColumn( TBColumnNew( 'Project', { || TimeData->ProjId } ) )
   o:AddColumn( TBColumnNew( 'Employee;Name', { || TimeData->EmplId } ) )
   o:AddColumn( TBColumnNew( 'Hours;Charged', { || TimeData->EmpHrReg } ) )
   o:AddColumn( TBColumnNew( 'Hours;ToGo', { || TimeData->ToGo } ) )

	@ 1,0 Clear to 1, maxcol()
   @ 1,27 say "Strike Shift - F7 to Print."

	// Print Information
   datadict( o, 'PAGEHEADER', 'Employee hours by Project' )

   datadict( o, "TB_TOTALS", .t. )      // Turn on Totaling

   // Control breaks set on Project Id and Employee Name
   datadict( o, "TB_BREAK", { 1, 2 } )

   // Define the subtotal line strings with embedded ## - these will be
   // replace with their current values at print time.
   // i.e. :
   //    Project:  SHIPPING
   //    Employee:  JOHN
   datadict( o, "TB_STRINGS", { "Grand Totals:  ", ;
                                "Project:  ##", "Employee:  ##" } )

   datadict( o:GetColumn( 1 ), "COL_BREAK", Nil )

   // Just for the heck of it - change the Nil to {|| Time() }
   datadict( o:GetColumn( 2 ), "COL_BREAK", Nil )

   datadict( o:GetColumn( 3 ), "COL_TOTAL", .t. )     // Total the 3rd column
   datadict( o:GetColumn( 3 ), "D", 1 )
   datadict( o:GetColumn( 4 ), "COL_TOTAL", .t. )     // Total the 4th column
   datadict( o:GetColumn( 4 ), "D", 1 )

   While .T.

      While NextKey() == 0 .and. ! o:stabilize()
      End
      nKey:= InKey( 0 )				// wait for a key

      DO CASE
         Case nKey == K_UP
            o:RefreshCurrent()
            o:up()
         Case nKey == K_DOWN
            o:RefreshCurrent()
             o:down()
         Case nKey == K_LEFT       ; o:left()
         Case nKey == K_RIGHT      ; o:right()
         Case nKey == K_CTRL_LEFT  ; o:panLeft()
         Case nKey == K_CTRL_RIGHT ; o:panRight()
         Case nKey == K_CTRL_PGUP
            o:RefreshCurrent()
            o:goTop()
         Case nKey == K_CTRL_PGDN
            o:RefreshCurrent()
            o:goBottom()
         Case nKey == K_PGUP
            o:RefreshCurrent()
            o:pageUp()
         Case nKey == K_PGDN
            o:RefreshCurrent()
            o:pageDown()
         Case nKey == K_HOME       ; o:home()
         Case nKey == K_END        ; o:end()
         Case nKey == K_CTRL_HOME  ; o:panHome()
         Case nKey == K_CTRL_END   ; o:panEnd()

         // HERE IT IS
         Case nKey == K_SH_F7      ; TBPrint( o )
         Case nKey == K_ALT_F7     ; TBPrint( o, .f., .t. )

         Case nKey == K_ESC        ; EXIT
      Endcase

   End

   cls

Return Nil

///////////////////////////////////////////////////////////////////////////
Function TBScopeDbf
   Local o, nKey, cLow, cHigh
   Field ProjId, EmplId

   Use TimeData New
   Index on ProjId + EmplId to TimeData
   Go Top

	o := TBrowseDB( 6, 10, 19, 69 )

   //Set up Filter Condition
   cLow = "ACCOUNT "
   cHigh = "ACCOUNT "

   // Scoped TBrowse Movement blocks!!!!!!
   // Skipper blocks Skipper(), SeekLow() and SeekHigh() are
   // coded at the end of this file.
   o:SkipBlock = { | n | Skipper( n, cLow, cHigh, { || TimeData->ProjId } ) }
   o:GoTopBlock = { || SeekLow( cLow ) }
   o:GoBottomBlock = { || SeekHigh( cHigh ) }

   Seek cLow

   o:HeadSep := ""
	o:ColSep  := " "

   o:AddColumn( TBColumnNew( 'Project', { || TimeData->ProjId } ) )
   o:AddColumn( TBColumnNew( 'Employee;Name', { || TimeData->EmplId } ) )
   o:AddColumn( TBColumnNew( 'Hours;Charged', { || TimeData->EmpHrReg } ) )
   o:AddColumn( TBColumnNew( 'Hours;ToGo', { || TimeData->ToGo } ) )

	@ 1,0 Clear to 1, maxcol()
   @ 1,27 say "Strike Shift - F7 to Print."

	// Print Information
   datadict( o, 'PAGEHEADER', 'Employee Time Data for ACCOUNT' )

   datadict( o, "TB_TOTALS", .t. )      // Turn on Totaling

   // Control breaks set on Project Id and Employee Name
   datadict( o, "TB_BREAK", { 2 } )

   datadict( o, "TB_STRINGS", { "Project Total:  ", ;
                                "Employee:  ##" } )

   datadict( o:GetColumn( 2 ), "COL_BREAK", Nil )

   datadict( o:GetColumn( 3 ), "COL_TOTAL", .t. )     // Total the 3rd column
   datadict( o:GetColumn( 3 ), "D", 1 )
   datadict( o:GetColumn( 4 ), "COL_TOTAL", .t. )     // Total the 4th column
   datadict( o:GetColumn( 4 ), "D", 1 )

   While .T.

      While NextKey() == 0 .and. ! o:stabilize()
      End
      nKey:= InKey( 0 )				// wait for a key

      DO CASE
         Case nKey == K_UP
            o:RefreshCurrent()
            o:up()
         Case nKey == K_DOWN
            o:RefreshCurrent()
             o:down()
         Case nKey == K_LEFT       ; o:left()
         Case nKey == K_RIGHT      ; o:right()
         Case nKey == K_CTRL_LEFT  ; o:panLeft()
         Case nKey == K_CTRL_RIGHT ; o:panRight()
         Case nKey == K_CTRL_PGUP
            o:RefreshCurrent()
            o:goTop()
         Case nKey == K_CTRL_PGDN
            o:RefreshCurrent()
            o:goBottom()
         Case nKey == K_PGUP
            o:RefreshCurrent()
            o:pageUp()
         Case nKey == K_PGDN
            o:RefreshCurrent()
            o:pageDown()
         Case nKey == K_HOME       ; o:home()
         Case nKey == K_END        ; o:end()
         Case nKey == K_CTRL_HOME  ; o:panHome()
         Case nKey == K_CTRL_END   ; o:panEnd()

         // HERE IT IS
         Case nKey == K_SH_F7      ; TBPrint( o )
         Case nKey == K_ALT_F7     ; TBPrint( o, .f., .t. )

         Case nKey == K_ESC        ; EXIT
      Endcase

   End

   cls

Return Nil

///////////////////////////////////////////////////////////////////////////
Function TBArray
	Local o, nKey, n := 1, nLen, aTasks := {}

	aadd( aTasks, { 'Andrew ', 'Brown ', 42, 'Engineer    ', 20 } )
	aadd( aTasks, { 'Dave   ', 'Brown ', 24, 'Carpenter   ', 2 } )
	aadd( aTasks, { 'Deloris', 'Blonde', 28, 'Dentist     ', 5 } )
	aadd( aTasks, { 'Frank  ', 'Brown ', 34, 'Soldier     ', 12 } )
	aadd( aTasks, { 'Greg   ', 'Black ', 27, 'Student     ', 22 } )
	aadd( aTasks, { 'James  ', 'Blonde', 30, 'Programmer  ', 6 } )
	aadd( aTasks, { 'Karen  ', 'Black ', 18, 'Student     ', 11 } )
	aadd( aTasks, { 'Lisa   ', 'Black ', 25, 'Lawyer      ', 3 } )
	aadd( aTasks, { 'Samual ', 'Grey  ', 54, 'Retired     ', 0 } )
	aadd( aTasks, { 'William', 'Red   ', 38, 'Truck Driver', 18 } )

	nLen = len( aTasks )
	o := TBrowseNew( 6, 10, 19, 69 )

	// Movement code blocks for array data
	o:goTopBlock := { || n:=1 }
	o:goBottomBlock := {|| n:=nLen }
  	o:skipBlock  :=  {|x,r| r:= IIf(x > 0, Min(x, nLen - n),	Max(x,-(n-1))),n+= r, r }

	o:HeadSep := ""
	o:ColSep  := " "

	o:AddColumn( TBColumnNew( 'First;Name', { || aTasks[ n, 1 ] } ) )
	o:AddColumn( TBColumnNew( ' Color;of Hair', { || aTasks[ n, 2 ] } ) )
	o:AddColumn( TBColumnNew( '       Age', { || aTasks[ n, 3 ] } ) )
	o:AddColumn( TBColumnNew( 'Occupation', { || aTasks[ n, 4 ] } ) )
	o:AddColumn( TBColumnNew( '     Years', { || aTasks[ n, 5 ] } ) )

	@ 1,0 Clear to 1, maxcol()
   @ 1, 27 say "Strike Shift - F7 to Print."

	// Print Information
	datadict( o, 'PAGEHEADER', 'Array Data for TBPrint' )
   datadict( o, "TB_TOTALS", .t. )      // Turn on Totaling

   datadict( o, "TB_STRINGS", { "Grand Total" } )

   // Go ahead and toggle these two and look at the results
   datadict( o:GetColumn( 3 ), "COL_TOTAL", .t. )
   datadict( o:GetColumn( 5 ), "COL_TOTAL", .f. )

   While .T.

      While NextKey() == 0 .and. ! o:stabilize()
      End

      nKey:= InKey( 0 )				// wait for a key
      DO CASE
         Case nKey == K_UP
            o:RefreshCurrent()
            o:up()
         Case nKey == K_DOWN
            o:RefreshCurrent()
             o:down()
         Case nKey == K_LEFT       ; o:left()
         Case nKey == K_RIGHT      ; o:right()
         Case nKey == K_CTRL_LEFT  ; o:panLeft()
         Case nKey == K_CTRL_RIGHT ; o:panRight()
         Case nKey == K_CTRL_PGUP
            o:RefreshCurrent()
            o:goTop()
         Case nKey == K_CTRL_PGDN
            o:RefreshCurrent()
            o:goBottom()
         Case nKey == K_PGUP
            o:RefreshCurrent()
            o:pageUp()
         Case nKey == K_PGDN
            o:RefreshCurrent()
            o:pageDown()
         Case nKey == K_HOME       ; o:home()
         Case nKey == K_END        ; o:end()
         Case nKey == K_CTRL_HOME  ; o:panHome()
         Case nKey == K_CTRL_END   ; o:panEnd()

         // HERE IT IS
         Case nKey == K_SH_F7      ; TBPrint( o )
         Case nKey == K_ALT_F7     ; TBPrint( o, .f., .t. )


         Case nKey == K_ESC        ; EXIT
      Endcase

   End

   cls

Return Nil

/////////////////////////////////////////////////////////
Function skipper( skipcount, low, high, block )
local  skipped := 0

if skipcount < 0         // skip backwards
      while ! bof() .and. skipped > skipcount .and. eval( block ) >= low
             skip -1
             skipped --
      end
      if bof()
      	go top
        skipped ++
      end
      // check to see that we didn't skip back one too many !!
      if eval( block ) < low
             skip
             skipped ++
      end
      if bof() .or. eval( block ) < low
             keyboard chr( K_ESC )    // break out of the browse - no data
      endif
end
if skipcount > 0     // skip forwards
      while ! eof() .and. skipped < skipcount .and. eval( block ) <= high
             skip
             skipped ++
      end
      // check to see that we didn't skip forward one too many !!
      if eof()
      	     skipped --
	     go bott
      endif
      if eval( block ) > high
             skip -1
             skipped --
      end
      if eof() .or. eval( block ) > high .or. eval( block ) < low
             keyboard chr( K_ESC )    // break out of the browse - no data
      endif
end
if skipcount = 0
      skip 0	// refresh underlying data source
end

Return skipped

////////////////////////////////////////////////////////////////////////
Function seekhigh( highval )
seek highval
if ! found()
	set softseek on
	seek highval
	set softseek off
	skip -1
end
return .t.

/////////////////////////////////////////////////////////////////////////
Function seeklow( lowval )
seek lowval
if ! found()
	set softseek on
	seek lowval
	set softseek off
end
return .t.

